@@ -260,6 +260,8 @@ static void
 xfce_appfinder_preferences_clear_history (XfceAppfinderPreferences *preferences)
 {
   XfceAppfinderModel *model;
+  gboolean            sort_by_frecency;
+  gint                scale_factor;
 
   appfinder_return_if_fail (XFCE_IS_APPFINDER_PREFERENCES (preferences));
 
@@ -267,7 +269,9 @@ xfce_appfinder_preferences_clear_history (XfceAppfinderPreferences *preferences)
                            _("This will permanently clear the custom command history."),
                            _("Are you sure you want to clear the command history?")))
     {
-      model = xfce_appfinder_model_get (xfconf_channel_get_bool (preferences->channel, "/sort-by-frecency", FALSE));
+      sort_by_frecency = xfconf_channel_get_bool (preferences->channel, "/sort-by-frecency", FALSE);
+      scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (preferences->dialog));
+      model = xfce_appfinder_model_get (sort_by_frecency, scale_factor);
       xfce_appfinder_model_history_clear (model);
       g_object_unref (G_OBJECT (model));
     }
