@@ -49,8 +49,8 @@
 
 
 
-#define DEFAULT_WINDOW_WIDTH   400
-#define DEFAULT_WINDOW_HEIGHT  400
+#define DEFAULT_WINDOW_WIDTH   580
+#define DEFAULT_WINDOW_HEIGHT  422
 #define DEFAULT_PANED_POSITION 180
 
 #define XFCE_APPFINDER_LOCAL_PREFIX "file://"
@@ -217,15 +217,20 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   GtkEntryCompletion *completion;
   GtkCellRenderer    *cell;
   gint                integer;
+  gboolean            sort_by_frecency;
+  gint                scale_factor;
+
+  scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
 
   window->channel = xfconf_channel_get ("xfce4-appfinder");
   window->last_window_height = xfconf_channel_get_int (window->channel, "/last/window-height", DEFAULT_WINDOW_HEIGHT);
 
-  window->category_model = xfce_appfinder_category_model_new ();
+  window->category_model = xfce_appfinder_category_model_new (scale_factor);
   xfconf_g_property_bind (window->channel, "/category-icon-size", G_TYPE_UINT,
                           G_OBJECT (window->category_model), "icon-size");
 
-  window->model = xfce_appfinder_model_get (xfconf_channel_get_bool (window->channel, "/sort-by-frecency", FALSE));
+  sort_by_frecency = xfconf_channel_get_bool (window->channel, "/sort-by-frecency", FALSE);
+  window->model = xfce_appfinder_model_get (sort_by_frecency, scale_factor);
   xfconf_g_property_bind (window->channel, "/item-icon-size", G_TYPE_UINT,
                           G_OBJECT (window->model), "icon-size");
 
@@ -234,10 +239,11 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_window_set_default_size (GTK_WINDOW (window), integer, -1);
   gtk_window_set_icon_name (GTK_WINDOW (window), "org.xfce.appfinder");
 
-  if (xfconf_channel_get_bool (window->channel, "/always-center", FALSE))
-    {
-      gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
-    }
+  if (xfconf_channel_get_bool (window->channel, "/always-center", TRUE))
+    gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+
+  if (xfconf_channel_get_bool (window->channel, "/hide-window-decorations", FALSE))
+    gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -248,7 +254,9 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show (hbox);
 
-  window->icon_find = xfce_appfinder_model_load_pixbuf (XFCE_APPFINDER_STOCK_FIND, XFCE_APPFINDER_ICON_SIZE_48);
+  window->icon_find = xfce_appfinder_model_load_pixbuf (XFCE_APPFINDER_ICON_NAME_FIND,
+                                                        XFCE_APPFINDER_ICON_SIZE_DEFAULT_ITEM,
+                                                        scale_factor);
   window->image = image = gtk_image_new_from_pixbuf (window->icon_find);
   gtk_widget_set_size_request (image, 48, 48);
   gtk_widget_set_halign(image, GTK_ALIGN_CENTER);
@@ -272,7 +280,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
       G_CALLBACK (xfce_appfinder_window_entry_key_press_event), window);
   gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
                                      GTK_ENTRY_ICON_SECONDARY,
-                                     XFCE_APPFINDER_STOCK_GO_DOWN);
+                                     XFCE_APPFINDER_ICON_NAME_GO_DOWN);
   gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry),
                                    GTK_ENTRY_ICON_SECONDARY,
                                    _("Toggle view mode"));
@@ -306,7 +314,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_widget_show (scroll);
 
-  if (xfconf_channel_get_bool (window->channel, "/hide-category-pane", FALSE))
+  if (xfconf_channel_get_bool (window->channel, "/hide-category-pane", TRUE))
     gtk_widget_set_visible (scroll, FALSE);
 
   sidepane = window->sidepane = gtk_tree_view_new_with_model (GTK_TREE_MODEL (window->category_model));
@@ -333,7 +341,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (column), renderer, FALSE);
   gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN (column), renderer,
-                                       "pixbuf", XFCE_APPFINDER_CATEGORY_MODEL_COLUMN_ICON, NULL);
+                                       "surface", XFCE_APPFINDER_CATEGORY_MODEL_COLUMN_ICON, NULL);
 
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -362,7 +370,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_button_set_always_show_image(GTK_BUTTON(button), TRUE);
   gtk_widget_show (button);
 
-  image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_PREFERENCES, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_PREFERENCES, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   window->button_launch = button = gtk_button_new_with_mnemonic (_("La_unch"));
@@ -373,14 +381,19 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_button_set_always_show_image(GTK_BUTTON(button), TRUE);
   gtk_widget_show (button);
 
-  image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_EXECUTE, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
+  /* update icons when icon theme changes */
   window->icon_theme = gtk_icon_theme_get_for_screen (gtk_window_get_screen (GTK_WINDOW (window)));
   g_signal_connect_swapped (G_OBJECT (window->icon_theme), "changed",
       G_CALLBACK (xfce_appfinder_window_icon_theme_changed), window);
   g_object_ref (G_OBJECT (window->icon_theme));
 
+  /* update icons when scale factor changes */
+  g_signal_connect (G_OBJECT (window), "notify::scale-factor",
+      G_CALLBACK (xfce_appfinder_window_icon_theme_changed), NULL);
+
   /* load categories in the model */
   xfce_appfinder_window_category_set_categories (NULL, window);
   window->categories_changed_id =
@@ -409,6 +422,9 @@ xfce_appfinder_window_finalize (GObject *object)
   g_signal_handler_disconnect (window->channel, window->property_watch_id);
   g_signal_handler_disconnect (window->model, window->categories_changed_id);
 
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window),
+      xfce_appfinder_window_icon_theme_changed, NULL);
+
   /* release our reference on the icon theme */
   g_signal_handlers_disconnect_by_func (G_OBJECT (window->icon_theme),
       xfce_appfinder_window_icon_theme_changed, window);
@@ -533,7 +549,7 @@ xfce_appfinder_window_window_state_event (GtkWidget           *widget,
 
           /* set sensible width instead of taking entire width */
           width = xfconf_channel_get_int (window->channel, "/last/window-width", DEFAULT_WINDOW_WIDTH);
-          gtk_window_resize (GTK_WINDOW (widget), width, 100 /* should be corrected by wm */);
+          gtk_window_resize (GTK_WINDOW (widget), width, 80 /* should be corrected by wm */);
         }
 
       if ((event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0)
@@ -694,14 +710,20 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       gtk_widget_destroy (window->view);
     }
 
-  window->filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (window->model), NULL);
-  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (window->filter_model), xfce_appfinder_window_item_visible, window, NULL);
+  if (window->filter_model == NULL)
+    {
+      window->filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (window->model), NULL);
+      gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (window->filter_model), xfce_appfinder_window_item_visible, window, NULL);
+    }
 
-  window->sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (window->filter_model));
-  if (xfconf_channel_get_bool (window->channel, "/sort-by-frecency", FALSE))
-    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->sort_model), xfce_appfinder_window_sort_items_frecency, window->entry, NULL);
-  else
-    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->sort_model), xfce_appfinder_window_sort_items, window->entry, NULL);
+  if (window->sort_model == NULL)
+    {
+      window->sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (window->filter_model));
+      if (xfconf_channel_get_bool (window->channel, "/sort-by-frecency", FALSE))
+        gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->sort_model), xfce_appfinder_window_sort_items_frecency, window->entry, NULL);
+      else
+        gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->sort_model), xfce_appfinder_window_sort_items, window->entry, NULL);
+    }
 
   if (icon_view)
     {
@@ -711,7 +733,7 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
           window->filter_model);
 
       gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (view), GTK_SELECTION_SINGLE);
-      gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_ICON);
+      gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_PIXBUF);
       gtk_icon_view_set_text_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TITLE);
       gtk_icon_view_set_tooltip_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TOOLTIP);
       gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (view), 0);
@@ -748,7 +770,7 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       renderer = gtk_cell_renderer_pixbuf_new ();
       gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (column), renderer, FALSE);
       gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (column), renderer,
-                                      "pixbuf", XFCE_APPFINDER_MODEL_COLUMN_ICON, NULL);
+                                      "surface", XFCE_APPFINDER_MODEL_COLUMN_ICON, NULL);
 
       renderer = gtk_cell_renderer_text_new ();
       g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -919,7 +941,7 @@ xfce_appfinder_window_popup_menu_revert (GtkWidget           *mi,
   if (name == NULL)
     return;
 
-  if (xfce_dialog_confirm (GTK_WINDOW (window), XFCE_APPFINDER_STOCK_REVERT_TO_SAVED, NULL,
+  if (xfce_dialog_confirm (GTK_WINDOW (window), XFCE_APPFINDER_ICON_NAME_REVERT, NULL,
           _("This will permanently remove the custom desktop file from your home directory."),
           _("Are you sure you want to revert \"%s\"?"), name))
     {
@@ -1070,9 +1092,9 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
           G_CALLBACK (xfce_appfinder_window_popup_menu_toggle_bookmark), window);
 
       if (is_bookmark)
-        image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+        image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_REMOVE, GTK_ICON_SIZE_MENU);
       else
-        image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_BOOKMARK_NEW, GTK_ICON_SIZE_MENU);
+        image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_BOOKMARK_NEW, GTK_ICON_SIZE_MENU);
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       label = gtk_label_new (is_bookmark ? _("Remove From Bookmarks") : _("Add to Bookmarks"));
@@ -1094,7 +1116,7 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       label = gtk_label_new_with_mnemonic (_("La_unch"));
-      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
+      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_EXECUTE, GTK_ICON_SIZE_MENU);
       gtk_container_add (GTK_CONTAINER (box), image);
       gtk_container_add (GTK_CONTAINER (box), label);
       gtk_container_add (GTK_CONTAINER (mi), box);
@@ -1108,7 +1130,7 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       label = gtk_label_new_with_mnemonic (_("_Edit"));
-      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_EDIT, GTK_ICON_SIZE_MENU);
+      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_EDIT, GTK_ICON_SIZE_MENU);
       gtk_container_add (GTK_CONTAINER (box), image);
       gtk_container_add (GTK_CONTAINER (box), label);
       gtk_container_add (GTK_CONTAINER (mi), box);
@@ -1125,7 +1147,7 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       label = gtk_label_new_with_mnemonic (_("_Revert"));
-      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_REVERT_TO_SAVED, GTK_ICON_SIZE_MENU);
+      image = gtk_image_new_from_icon_name (XFCE_APPFINDER_ICON_NAME_REVERT, GTK_ICON_SIZE_MENU);
       gtk_container_add (GTK_CONTAINER (box), image);
       gtk_container_add (GTK_CONTAINER (box), label);
       gtk_container_add (GTK_CONTAINER (mi), box);
@@ -1158,14 +1180,23 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
 
 static void
 xfce_appfinder_window_update_image (XfceAppfinderWindow *window,
-                                    GdkPixbuf           *pixbuf)
+                                    cairo_surface_t     *surface)
 {
-  if (pixbuf == NULL)
-    pixbuf = window->icon_find;
+  gint     scale_factor;
+  gboolean free_surface = FALSE;
 
-  /* gtk doesn't check this */
-  if (gtk_image_get_pixbuf (GTK_IMAGE (window->image)) != pixbuf)
-    gtk_image_set_from_pixbuf (GTK_IMAGE (window->image), pixbuf);
+  if (surface == NULL)
+    {
+      scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
+      surface = gdk_cairo_surface_create_from_pixbuf (window->icon_find, scale_factor,
+                                                      gtk_widget_get_window (GTK_WIDGET (window)));
+      free_surface = TRUE;
+    }
+
+    gtk_image_set_from_surface (GTK_IMAGE (window->image), surface);
+
+    if (free_surface)
+      cairo_surface_destroy (surface);
 }
 
 
@@ -1214,7 +1245,7 @@ xfce_appfinder_window_entry_changed_idle (gpointer data)
 {
   XfceAppfinderWindow *window = XFCE_APPFINDER_WINDOW (data);
   const gchar         *text;
-  GdkPixbuf           *pixbuf;
+  cairo_surface_t     *surface;
   gchar               *normalized;
   GtkTreePath         *path;
   GtkTreeSelection    *selection;
@@ -1287,10 +1318,10 @@ xfce_appfinder_window_entry_changed_idle (gpointer data)
       gtk_entry_set_icon_from_icon_name (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY, NULL);
       gtk_entry_set_icon_tooltip_text (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY, NULL);
 
-      pixbuf = xfce_appfinder_model_get_icon_for_command (window->model, text);
-      xfce_appfinder_window_update_image (window, pixbuf);
-      if (pixbuf != NULL)
-        g_object_unref (G_OBJECT (pixbuf));
+      surface = xfce_appfinder_model_get_icon_for_command (window->model, text);
+      xfce_appfinder_window_update_image (window, surface);
+      if (surface != NULL)
+        cairo_surface_destroy (surface);
     }
 
 
@@ -1427,22 +1458,22 @@ xfce_appfinder_window_drag_begin (GtkWidget           *widget,
                                   GdkDragContext      *drag_context,
                                   XfceAppfinderWindow *window)
 {
-  GtkTreeModel *model;
-  GtkTreeIter   iter;
-  GdkPixbuf    *pixbuf;
+  GtkTreeModel    *model;
+  GtkTreeIter      iter;
+  cairo_surface_t *surface;
 
   if (xfce_appfinder_window_view_get_selected (window, &model, &iter))
     {
-      gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_ICON_LARGE, &pixbuf, -1);
-      if (G_LIKELY (pixbuf != NULL))
+      gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_ICON_LARGE, &surface, -1);
+      if (G_LIKELY (surface != NULL))
         {
-          gtk_drag_set_icon_pixbuf (drag_context, pixbuf, 0, 0);
-          g_object_unref (G_OBJECT (pixbuf));
+          gtk_drag_set_icon_surface (drag_context, surface);
+          cairo_surface_destroy (surface);
         }
     }
   else
     {
-      gtk_drag_set_icon_name (drag_context, XFCE_APPFINDER_STOCK_DIALOG_ERROR, 0, 0);
+      gtk_drag_set_icon_name (drag_context, XFCE_APPFINDER_ICON_NAME_DIALOG_ERROR, 0, 0);
     }
 }
 
@@ -1682,6 +1713,11 @@ xfce_appfinder_window_property_changed (XfconfChannel       *channel,
         gtk_tree_path_free (path);
       }
     }
+  else if (g_strcmp0 (prop, "/hide-window-decorations") == 0)
+    {
+      gboolean hide = g_value_get_boolean (value);
+      gtk_window_set_decorated (GTK_WINDOW (window), !hide);
+    }
 }
 
 
@@ -1709,7 +1745,7 @@ xfce_appfinder_window_item_changed (XfceAppfinderWindow *window)
   GtkTreeIter       iter;
   GtkTreeModel     *model;
   gboolean          can_launch;
-  GdkPixbuf        *pixbuf;
+  cairo_surface_t  *surface;
 
   if (gtk_widget_get_visible (window->paned))
     {
@@ -1718,11 +1754,11 @@ xfce_appfinder_window_item_changed (XfceAppfinderWindow *window)
 
       if (can_launch)
         {
-          gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_ICON_LARGE, &pixbuf, -1);
-          if (G_LIKELY (pixbuf != NULL))
+          gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_ICON_LARGE, &surface, -1);
+          if (G_LIKELY (surface != NULL))
             {
-              xfce_appfinder_window_update_image (window, pixbuf);
-              g_object_unref (G_OBJECT (pixbuf));
+              xfce_appfinder_window_update_image (window, surface);
+              cairo_surface_destroy (surface);
             }
         }
       else
@@ -1746,18 +1782,30 @@ xfce_appfinder_window_row_activated (XfceAppfinderWindow *window)
 static void
 xfce_appfinder_window_icon_theme_changed (XfceAppfinderWindow *window)
 {
+  gint scale_factor;
+
   appfinder_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
 
+  scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
+
   if (window->icon_find != NULL)
     g_object_unref (G_OBJECT (window->icon_find));
-  window->icon_find = xfce_appfinder_model_load_pixbuf (XFCE_APPFINDER_STOCK_FIND, XFCE_APPFINDER_ICON_SIZE_48);
+  window->icon_find = xfce_appfinder_model_load_pixbuf (XFCE_APPFINDER_ICON_NAME_FIND,
+                                                        XFCE_APPFINDER_ICON_SIZE_DEFAULT_ITEM,
+                                                        scale_factor);
 
-  /* drop cached pixbufs */
+  /* drop cached pixbufs and surfaces */
   if (G_LIKELY (window->model != NULL))
-    xfce_appfinder_model_icon_theme_changed (window->model);
+    {
+      g_object_set (window->model, "scale-factor", scale_factor, NULL);
+      xfce_appfinder_model_icon_theme_changed (window->model);
+    }
 
   if (G_LIKELY (window->category_model != NULL))
-    xfce_appfinder_category_model_icon_theme_changed (window->category_model);
+    {
+      g_object_set (window->category_model, "scale-factor", scale_factor, NULL);
+      xfce_appfinder_category_model_icon_theme_changed (window->category_model);
+    }
 
   /* update state */
   xfce_appfinder_window_entry_changed (window);
@@ -1798,7 +1846,7 @@ xfce_appfinder_window_execute_command (const gchar          *text,
 
   if (IS_STRING (text))
     {
-      gchar **argv;
+      gchar **argv = NULL;
       gchar **envp;
 
       /* expand variables */
@@ -1916,7 +1964,7 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window,
   if (!only_custom_cmd)
     {
       gtk_entry_set_icon_from_icon_name (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
-                                     result ? NULL : XFCE_APPFINDER_STOCK_DIALOG_ERROR);
+                                     result ? NULL : XFCE_APPFINDER_ICON_NAME_DIALOG_ERROR);
       gtk_entry_set_icon_tooltip_text (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
                                        error != NULL ? error->message : NULL);
     }
@@ -1963,27 +2011,25 @@ void
 xfce_appfinder_window_set_expanded (XfceAppfinderWindow *window,
                                     gboolean             expanded)
 {
-  GdkGeometry         hints;
   gint                width;
   GtkEntryCompletion *completion;
 
   APPFINDER_DEBUG ("set expand = %s", expanded ? "true" : "false");
 
-  /* force window geomentry */
+  /* update window geometry */
   if (expanded)
     {
-      gtk_window_set_geometry_hints (GTK_WINDOW (window), NULL, NULL, 0);
       gtk_window_get_size (GTK_WINDOW (window), &width, NULL);
       gtk_window_resize (GTK_WINDOW (window), width, window->last_window_height);
     }
   else
     {
       if (gtk_widget_get_visible (GTK_WIDGET (window)))
-        gtk_window_get_size (GTK_WINDOW (window), NULL, &window->last_window_height);
+        gtk_window_get_size (GTK_WINDOW (window), &width, &window->last_window_height);
+      else
+        width = xfconf_channel_get_int (window->channel, "/last/window-width", DEFAULT_WINDOW_WIDTH);
 
-      hints.max_height = -1;
-      hints.max_width = G_MAXINT;
-      gtk_window_set_geometry_hints (GTK_WINDOW (window), NULL, &hints, GDK_HINT_MAX_SIZE);
+      gtk_window_resize (GTK_WINDOW (window), width, 80 /* should be corrected by wm */);
     }
 
   /* show/hide pane with treeviews */
@@ -1991,7 +2037,7 @@ xfce_appfinder_window_set_expanded (XfceAppfinderWindow *window,
 
   /* toggle icon */
   gtk_entry_set_icon_from_icon_name (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_SECONDARY,
-                                     expanded ? XFCE_APPFINDER_STOCK_GO_UP : XFCE_APPFINDER_STOCK_GO_DOWN);
+                                     expanded ? XFCE_APPFINDER_ICON_NAME_GO_UP : XFCE_APPFINDER_ICON_NAME_GO_DOWN);
   gtk_entry_set_icon_from_icon_name (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY, NULL);
 
   /* update completion (remove completed text of restart completion) */
